@@ -1,10 +1,17 @@
+import abc
 import atexit
 import contextlib
+import contextvars
+import functools
+import inspect
 import os
 import math
 import platform
 import signal
+import textwrap
+import threading
 import typing
+import warnings
 from typing import Union
 from rpy2.rinterface_lib import openrlib
 import rpy2.rinterface_lib._rinterface_capi as _rinterface
@@ -13,6 +20,7 @@ import rpy2.rinterface_lib.conversion as conversion
 from rpy2.rinterface_lib.conversion import _cdata_res_to_rinterface
 import rpy2.rinterface_lib.memorymanagement as memorymanagement
 from rpy2.rinterface_lib import na_values
+from rpy2.rinterface_lib.sexp import NULL
 from rpy2.rinterface_lib.sexp import NULLType
 import rpy2.rinterface_lib.bufferprotocol as bufferprotocol
 from rpy2.rinterface_lib import sexp
@@ -35,12 +43,16 @@ R_NilValue = openrlib.rlib.R_NilValue
 
 endr = embedded.endr
 
-_evaluation_context = globalenv
+evaluation_context = contextvars.ContextVar('evaluation_context',
+                                            default=globalenv)
 
 
 def get_evaluation_context() -> SexpEnvironment:
     """Get the frame (environment) in which R code is currently evaluated."""
-    return _evaluation_context
+    warnings.warn('This function is deprecated. '
+                  'Use evaluation_context directly.',
+                  DeprecationWarning, stacklevel=2)
+    return evaluation_context.get()
 
 
 @contextlib.contextmanager
@@ -60,9 +72,7 @@ def local_context(
     Yield the environment (passed to env, or created).
     """
 
-    global _evaluation_context
-
-    parent_frame = _evaluation_context
+    parent_frame = evaluation_context.get()
     if env is None:
         env = baseenv['new.env'](
             baseenv['parent.frame']()
@@ -71,13 +81,13 @@ def local_context(
     try:
         if use_rlock:
             with openrlib.rlock:
-                _evaluation_context = env
+                token = evaluation_context.set(env)
                 yield env
         else:
-            _evaluation_context = env
+            token = evaluation_context.set(env)
             yield env
     finally:
-        _evaluation_context = parent_frame
+        evaluation_context.reset(token)
 
 
 def _sigint_handler(sig, frame):
@@ -88,8 +98,8 @@ def _sigint_handler(sig, frame):
 def parse(text: str, num: int = -1):
     """Parse a string as R code.
 
-    :param:`text` A string with R code to parse.
-    :param:`num` The maximum number of lines to parse. If -1, no
+    :param text: A string with R code to parse.
+    :param num: The maximum number of lines to parse. If -1, no
       limit is applied.
     """
 
@@ -101,28 +111,79 @@ def parse(text: str, num: int = -1):
     return res
 
 
-def evalr(source: str, maxlines: int = -1) -> sexp.Sexp:
+def evalr_expr(
+        expr: 'ExprSexpVector',
+        envir: typing.Union[
+            None,
+            'SexpEnvironment', 'NULLType',
+            'ListSexpVector', 'PairlistSexpVector', int,
+            '_MissingArgType'] = None,
+        enclos: typing.Union[
+            None,
+            'ListSexpVector', 'PairlistSexpVector',
+            'NULLType',
+            '_MissingArgType'] = None
+) -> sexp.Sexp:
+    """Evaluate an R expression.
+
+    :param expr: An R expression.
+    :param envir: An optional R environment in which the evaluation
+      will happen. If None, which is the default, the environment in
+     the context variable `evaluation_context` is used.
+    :param enclos: An enclosure. This is only relevant when envir
+      is a list, a pairlist, or a data.frame. It specifies where to
+    look for objects not found in envir.
+    :return: The R objects resulting from the evaluation of the code"""
+
+    if envir is None:
+        envir = evaluation_context.get()
+    if enclos is None:
+        enclos = MissingArg
+    res = baseenv['eval'](expr, envir=envir, enclos=enclos)
+    return res
+
+
+def evalr(
+        source: str,
+        maxlines: int = -1,
+        envir: typing.Union[
+            None,
+            'SexpEnvironment', 'NULLType',
+            'ListSexpVector', 'PairlistSexpVector', int,
+            '_MissingArgType'] = None,
+        enclos: typing.Union[
+            None,
+            'ListSexpVector', 'PairlistSexpVector',
+            'NULLType', '_MissingArgType'] = None
+) -> sexp.Sexp:
     """Evaluate a string as R code.
 
     Evaluate a string as R just as it would happen when writing
     code in an R terminal.
 
-    :param:`text` A string to be evaluated as R code.
-    :param:`maxlines` The maximum number of lines to parse. If -1, no
-      limit is applied."""
+    :param str text: A string to be evaluated as R code,
+    or an R expression.
+    :param int maxlines: The maximum number of lines to parse. If -1, no
+      limit is applied.
+    :param envir: An optional R environment in which the evaluation
+      will happen.
+    :param enclos: An enclosure. This is only relevant when envir
+      is a list, a pairlist, or a data.frame. It specifies where to
+    look for objects not found in envir.
+    :return: The R objects resulting from the evaluation of the code"""
 
-    res = parse(source, num=maxlines)
-    res = baseenv['eval'](res)
+    expr = parse(source, num=maxlines)
+    res = evalr_expr(expr, envir=envir, enclos=enclos)
     return res
 
 
 def vector_memoryview(obj: sexp.SexpVector,
                       sizeof_str: str, cast_str: str) -> memoryview:
     """
-    :param:`obj` R vector
-    :param:`sizeof_str` Type in a string to use with ffi.sizeof()
+    :param obj: R vector
+    :param str sizeof_str: Type in a string to use with ffi.sizeof()
         (for example "int")
-    :param:`cast_str` Type in a string to use with memoryview.cast()
+    :param str cast_str: Type in a string to use with memoryview.cast()
         (for example "i")
     """
     b = openrlib.ffi.buffer(
@@ -135,13 +196,21 @@ def vector_memoryview(obj: sexp.SexpVector,
     # mv = memoryview(b).cast(cast_str, shape, order='F')
     # ```
     # but Python does not handle FORTRAN-ordered arrays without having
-    # to write C extensions. We have to use numpy.
-    # TODO: Having numpy a requirement just for this is a problem.
-    # TODO: numpy needed for memoryview
-    #   (as long as https://bugs.python.org/issue34778 not resolved)
-    import numpy
-    a = numpy.frombuffer(b, dtype=cast_str).reshape(shape, order='F')
-    mv = memoryview(a)
+    # to write C extensions (see
+    # https://bugs.python.org/issue34778 is not resolved).
+    # Having numpy a requirement just for this is a problem so a
+    # C function to swap the strides in place was written
+    try:
+        import rpy2.rinterface_lib._bufferprotocol as bp   # type: ignore
+        mv = memoryview(b).cast(cast_str, shape)
+        bp.memoryview_swapstrides(mv)
+    except ImportError:
+        import numpy  # type: ignore
+        a = numpy.frombuffer(b, dtype=cast_str).reshape(shape, order='F')
+        # The typed signature for memoryview does not help the static
+        # type checker verify that a numpy.ndarray implement the buffer
+        # protocol. Type checking is ignored to avoid a check error.
+        mv = memoryview(a)  # type: ignore
     return mv
 
 
@@ -177,6 +246,11 @@ class SexpSymbol(sexp.Sexp):
 
 
 class _MissingArgType(SexpSymbol, metaclass=sexp.SingletonABC):
+    """Missing argument in a call to an R function.
+
+    When evaluating a function, arguments that are not specified
+    can take a default value when named, otherwise they will
+    take a special value that indicates that they are missing."""
 
     def __init__(self):
         if embedded.isready():
@@ -196,9 +270,15 @@ class _MissingArgType(SexpSymbol, metaclass=sexp.SingletonABC):
         return False
 
     @property
-    def __sexp__(self) -> _rinterface.SexpCapsule:
+    def __sexp__(self) -> typing.Union['_rinterface.SexpCapsule',
+                                       '_rinterface.UninitializedRCapsule']:
         return self._sexpobject
 
+    @__sexp__.setter
+    def __sexp__(self, value: typing.Union['_rinterface.SexpCapsule',
+                                           '_rinterface.UninitializedRCapsule']) -> None:
+        raise TypeError('The capsule for the R object cannot be modified.')
+
 
 MissingArg = _MissingArgType()
 
@@ -209,7 +289,7 @@ class SexpPromise(Sexp):
     def eval(self, env: typing.Optional[SexpEnvironment] = None) -> sexp.Sexp:
         """"Evalute the R "promise".
 
-        :param:`env` The environment in which to evaluate the
+        :param env: The environment in which to evaluate the
           promise.
         """
         if env is None:
@@ -217,22 +297,20 @@ class SexpPromise(Sexp):
         return openrlib.rlib.Rf_eval(self.__sexp__._cdata, env)
 
 
-NPCOMPAT_TYPE = typing.TypeVar('NPCOMPAT_TYPE',
-                               'ByteSexpVector',
-                               'BoolSexpVector',
-                               'IntSexpVector',
-                               'FloatSexpVector')
-
-
-class NumpyArrayMixin:
+class SexpVectorWithNumpyInterface(SexpVector):
     """Numpy-specific API for accessing the content of a numpy array.
 
     This interface implements version 3 of Numpy's `__array_interface__`
     and is only available / possible for some of the R vectors."""
 
+    @property
+    @abc.abstractmethod
+    def _NP_TYPESTR(self) -> str:
+        pass
+
     @property
     def __array_interface__(
-            self: NPCOMPAT_TYPE
+            self
     ) -> dict:
         """Return an `__array_interface__` version 3.
 
@@ -252,8 +330,20 @@ class NumpyArrayMixin:
                 'data': data,
                 'version': 3}
 
+    @classmethod
+    def _check_C_compatible(cls, mview):
+        return (
+            mview.itemsize == cls._R_SIZEOF_ELT
+            and
+            (
+                cls._NP_TYPESTR == '|u1'
+                or
+                cls._NP_TYPESTR.endswith(mview.format)
+            )
+        )
 
-class ByteSexpVector(NumpyArrayMixin, SexpVector):
+
+class ByteSexpVector(SexpVectorWithNumpyInterface):
     """Array of bytes.
 
     This is the R equivalent to a Python :class:`bytesarray`.
@@ -322,7 +412,7 @@ class ByteSexpVector(NumpyArrayMixin, SexpVector):
                 'Indices must be integers or slices, not %s' % type(i))
 
 
-class BoolSexpVector(NumpyArrayMixin, SexpVector):
+class BoolSexpVector(SexpVectorWithNumpyInterface):
     """Array of booleans.
 
     Note that R is internally storing booleans as integers to
@@ -342,13 +432,18 @@ class BoolSexpVector(NumpyArrayMixin, SexpVector):
         else:
             return bool(x)
 
-    def __getitem__(self, i: Union[int, slice]) -> Union[typing.Optional[bool],
+    def __getitem__(self, i: Union[int, slice]) -> Union[bool,
+                                                         'sexp.NALogicalType',
                                                          'BoolSexpVector']:
+        res: Union[bool,
+                   'sexp.NALogicalType',
+                   'BoolSexpVector']
         cdata = self.__sexp__._cdata
         if isinstance(i, int):
             i_c = _rinterface._python_index_to_c(cdata, i)
             elt = openrlib.LOGICAL_ELT(cdata, i_c)
-            res = na_values.NA_Logical if elt == NA_Logical else bool(elt)
+            res = (na_values.NA_Logical  # type: ignore
+                   if elt == NA_Logical else bool(elt))
         elif isinstance(i, slice):
             res = type(self).from_iterable(
                 [openrlib.LOGICAL_ELT(cdata, i_c)
@@ -384,7 +479,7 @@ def nullable_int(v):
         return int(v)
 
 
-class IntSexpVector(NumpyArrayMixin, SexpVector):
+class IntSexpVector(SexpVectorWithNumpyInterface):
 
     _R_TYPE = openrlib.rlib.INTSXP
     _R_SET_VECTOR_ELT = openrlib.SET_INTEGER_ELT
@@ -431,7 +526,7 @@ class IntSexpVector(NumpyArrayMixin, SexpVector):
         return vector_memoryview(self, 'int', 'i')
 
 
-class FloatSexpVector(NumpyArrayMixin, SexpVector):
+class FloatSexpVector(SexpVectorWithNumpyInterface):
 
     _R_TYPE = openrlib.rlib.REALSXP
     _R_VECTOR_ELT = openrlib.REAL_ELT
@@ -632,7 +727,31 @@ class ExprSexpVector(SexpVector):
     _R_SET_VECTOR_ELT = None
 
 
+_str2lang = None
+
+
+def _get_str2lang():
+    global _str2lang
+    if _str2lang is None:
+        try:
+            _str2lang = baseenv['str2lang']
+        except KeyError:
+            # TODO: This exists to ensure compatibility with
+            # older versions of R. If should be removed when
+            # older versions of R are no longer supported.
+            _str2lang = evalr('function(s) '
+                              'base::parse(text=s, keep.source=FALSE)[[1]]')
+    return _str2lang
+
+
+LangSexpVector_VT = typing.TypeVar('LangSexpVector_VT', bound='LangSexpVector')
+
+
 class LangSexpVector(SexpVector):
+    """An R language object.
+
+    To create from a (Python) string containing R code
+    use the classmethod `from_string`."""
     _R_TYPE = openrlib.rlib.LANGSXP
     _R_GET_PTR = None
     _CAST_IN = None
@@ -648,7 +767,12 @@ class LangSexpVector(SexpVector):
             openrlib.rlib.Rf_nthcdr(cdata, i_c)
         )
 
-    def __setitem__(self, i: int, value: sexp.SupportsSEXP) -> None:
+    def __setitem__(self, i: typing.Union[int, slice],
+                    value: sexp.SupportsSEXP) -> None:
+        if isinstance(i, slice):
+            raise NotImplementedError(
+                'Assigning slices to LangSexpVectors is not yet implemented.'
+            )
         cdata = self.__sexp__._cdata
         i_c = _rinterface._python_index_to_c(cdata, i)
         openrlib.rlib.SETCAR(
@@ -656,6 +780,22 @@ class LangSexpVector(SexpVector):
             value.__sexp__._cdata
         )
 
+    @classmethod
+    def from_string(
+            cls: typing.Type[LangSexpVector_VT], s: str
+    ) -> LangSexpVector_VT:
+        """Create an R language object from a string.
+
+        This creates an unevaluated R language object.
+
+        Args:
+            s: R source code in a string.
+
+        Returns:
+            An instance of the class the method is called from.
+        """
+        return cls(_get_str2lang()(s))
+
 
 class SexpClosure(Sexp):
 
@@ -666,7 +806,7 @@ class SexpClosure(Sexp):
             call_r = rmemory.protect(
                 _rinterface.build_rcall(self.__sexp__._cdata, args,
                                         kwargs.items()))
-            call_context = _evaluation_context
+            call_context = evaluation_context.get()
             res = rmemory.protect(
                 openrlib.rlib.R_tryEval(
                     call_r,
@@ -692,7 +832,7 @@ class SexpClosure(Sexp):
         """
         # TODO: check keyvals are pairs ?
         if environment is None:
-            environment = _evaluation_context
+            environment = evaluation_context.get()
         assert isinstance(environment, SexpEnvironment)
         error_occured = _rinterface.ffi.new('int *', 0)
 
@@ -708,7 +848,7 @@ class SexpClosure(Sexp):
                 raise embedded.RRuntimeError(_rinterface._geterrmessage())
         return res
 
-    @property
+    @property  # type: ignore
     @_cdata_res_to_rinterface
     def closureenv(self) -> SexpEnvironment:
         """Closure of the R function."""
@@ -716,7 +856,10 @@ class SexpClosure(Sexp):
 
 
 class SexpS4(Sexp):
-    """R "S4" object."""
+    """R "S4" object.
+
+    S4 objects as one of the available forms of Object-Oriented Programming
+    in R."""
     pass
 
 
@@ -748,7 +891,7 @@ def make_extptr(obj, tag, protected):
 
 
 class SexpExtPtr(Sexp):
-
+    """R "External Pointer" object."""
     TYPE_TAG = 'Python'
 
     @classmethod
@@ -840,7 +983,6 @@ class RRuntimeWarning(RuntimeWarning):
     pass
 
 
-NULL = None
 MissingArg = _MissingArgType()
 NA_Character = None
 NA_Integer = None
@@ -860,7 +1002,21 @@ def initr_simple() -> typing.Optional[int]:
         return status
 
 
-def initr_checkenv() -> typing.Optional[int]:
+def initr_checkenv(
+        interactive: typing.Optional[bool] = None,
+        _want_setcallbacks: bool = True,
+        _c_stack_limit: typing.Optional[int] = None
+) -> typing.Optional[int]:
+    """Initialize the embedded R.
+
+    :param interactive: Should R run in interactive or non-interactive mode?
+    if `None` the value in `_DEFAULT_R_INTERACTIVE` will be used.
+    :param _want_setcallbacks: Should custom rpy2 callbacks for R frontends
+    be set?.
+    :param _c_stack_limit: Limit for the C Stack.
+    if `None` the value in `_DEFAULT_C_STACK_LIMIT` will be used.
+    """
+
     # Force the internal initialization flag if there is an environment
     # variable that indicates that R was already initialized in the current
     # process.
@@ -870,7 +1026,9 @@ def initr_checkenv() -> typing.Optional[int]:
         if embedded.is_r_externally_initialized():
             embedded._setinitialized()
         else:
-            status = embedded._initr()
+            status = embedded._initr(interactive=interactive,
+                                     _want_setcallbacks=_want_setcallbacks,
+                                     _c_stack_limit=_c_stack_limit)
             embedded.set_python_process_info()
         _rinterface._register_external_symbols()
         _post_initr_setup()
@@ -904,9 +1062,9 @@ def _post_initr_setup() -> None:
     emptyenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_EmptyEnv)
     baseenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_BaseEnv)
     globalenv.__sexp__ = _rinterface.SexpCapsule(openrlib.rlib.R_GlobalEnv)
-
-    global NULL
-    NULL = NULLType()
+    NULL._sexpobject = _rinterface.UnmanagedSexpCapsule(
+        openrlib.rlib.R_NilValue
+    )
 
     MissingArg._sexpobject = _rinterface.UnmanagedSexpCapsule(
         openrlib.rlib.R_MissingArg
@@ -937,29 +1095,198 @@ def _post_initr_setup() -> None:
     )
     NA_Complex = na_values.NA_Complex
 
-    signal.signal(signal.SIGINT, _sigint_handler)
+    warn_about_thread = False
+    if threading.current_thread() is threading.main_thread():
+        try:
+            signal.signal(signal.SIGINT, _sigint_handler)
+        except ValueError as ve:
+            if str(ve) == 'signal only works in main thread':
+                warn_about_thread = True
+            else:
+                raise ve
+    else:
+        warn_about_thread = True
+    if warn_about_thread:
+        warnings.warn(
+            textwrap.dedent(
+                """R is not initialized by the main thread.
+                Its taking over SIGINT cannot be reversed here, and as a
+                consequence the embedded R cannot be interrupted with Ctrl-C.
+                Consider (re)setting the signal handler of your choice from
+                the main thread."""
+            )
+        )
+
     _update_R_ENC_PY()
 
 
-def rternalize(function: typing.Callable) -> SexpClosure:
+def _find_first(nodes, of_type):
+    for node in nodes:
+        if isinstance(node, of_type):
+            return node
+    raise ValueError(
+        f'No node of type {of_type}'
+    )
+
+
+def rternalize(
+        function: typing.Optional[typing.Callable] = None, *,
+        signature: bool = False
+) -> typing.Union[SexpClosure, functools.partial]:
     """ Make a Python function callable from R.
 
     Takes an arbitrary Python function and wrap it in such a way that
     it can be called from the R side.
 
-    :param function: A Python callable object.
+    This factory can be used as a decorator, and has an optional
+    named argument "signature" that can be True or False (default is
+    False). When True, the R function wrapping the Python one will
+    have a matching signature or a close one, as detailed below.
+
+    The Python ellipsis arguments`*args` and `**kwargs` are
+    translated to the R ellipsis `...`.
+
+    For example:
+
+    .. code-block:: python
+       @rternalize(signature=True)
+       def foo(x, *args, y=2):
+           pass
+
+    will be visible in R with the signature:
+
+    .. code-block:: r
+       function (x, ..., y)
+
+    The only limitation is that whenever `*args` and `**kwargs` are
+    both present in the Python declaration they must be consecutive.
+    For example:
+
+    .. code-block:: python
+       def foo(x, *args, y=2, **kwargs):
+           pass
+
+    is a valid Python declaration. However, it cannot be "rternalized"
+    because the R ellipsis can only appear at most once in the signature
+    of a given function. Trying to apply the decorator `rternalize` would
+    raise an exception.
+
+    The following Python function can be "rternalized":
+
+    .. code-block:: python
+       def foo(x, *args, **kwargs):
+           pass
+
+    It is visible to R with the signature
+
+    .. code-block:: r
+       function (x, ...)
+
+    Python function definitions can allow the optional naming of required
+    arguments. The mapping of signatures between Python and R is then
+    quasi-indentical since R does it for unnamed arguments. The check
+    that all arguments are present is still performed on the Python side.
+
+    Example:
+
+    .. code-block:: python
+       @rternalize(signature=True)
+       def foo(x, *, y, z):
+           print(f'x: {x[0]}, y: {y[0]}, z: {z[0]}')
+           return ri.NULL
+
+    >>> _ = foo(1, 2, 3)
+    x: 1, y: 2, z: 3
+    ValueErro: None
+    >>> _ = foo(1)
+    TypeError: rternalized foo() missing 2 required keyword-only arguments: 'y' and 'z'
+    >>> _ = foo(1, z=2, y=3)
+    x: 1, y: 3, z: 2
+    >>> _ = foo(1, z=2, y=3)
+    x: 1, y: 3, z: 2
+
+    Note that whenever the Python function has an ellipsis (either `*args`
+    or `**kwargs`) earlier parameters in the signature that are
+    positional-or-keyword are considered to be positional arguments in a
+    function call.
+
+    :param function: A Python callable object. This is a positional
+    argument with a default value `None` to allow the decorator function
+    without parentheses when optional argument is not wanted.
+
     :return: A wrapped R object that can be use like any other rpy2
     object.
     """
+    if not embedded.isinitialized():
+        raise embedded.RNotReadyError('The embedded R is not yet initialized.')
+
+    if function is None:
+        return functools.partial(rternalize, signature=signature)
 
     assert callable(function)
     rpy_fun = SexpExtPtr.from_pyobject(function)
-    # TODO: this is a hack. Find a better way.
-    template = parse("""
-      function(...) { .External(".Python", foo, ...);
-    }
-    """)
-    template[0][2][1][2] = rpy_fun
+
+    if not signature:
+        template = parse("""
+        function(...) { .External(".Python", foo, ...);
+        }
+        """)
+        template[0][2][1][2] = rpy_fun
+    else:
+        has_ellipsis = None
+        params_r_sig = []
+        for p_i, (name, param) in enumerate(
+                inspect.signature(function).parameters.items()
+        ):
+            if (
+                    param.kind is inspect.Parameter.VAR_POSITIONAL or
+                    param.kind is inspect.Parameter.VAR_KEYWORD
+            ):
+                if has_ellipsis:
+                    if has_ellipsis != (p_i - 1):
+                        raise ValueError(
+                            'R functions can only have one ellipsis. '
+                            'As consequence your Python function must have *args '
+                            'and **kwargs that are consecutive in function '
+                            'signature.'
+                        )
+                else:
+                    # The ellipsis can only be added once.
+                    has_ellipsis = p_i
+                    params_r_sig.append('...')
+            else:
+                params_r_sig.append(name)
+
+        r_func_args = ', '.join(params_r_sig)
+
+        r_src = f"""
+        function({r_func_args}) {{
+            py_func <- RPY2_FUN_PLACEHOLDER
+            lst_args <- base::as.list(base::match.call()[-1])
+            RPY2_ARGUMENTS <- base::c(
+                base::list(
+                    ".Python",
+                    py_func
+                ),
+                lst_args
+            )
+            res <- base::do.call(
+               base::.External,
+               RPY2_ARGUMENTS
+            );
+
+            res
+        }}
+        """
+        template = parse(r_src)
+
+        function_definition = _find_first(template, of_type=LangSexpVector)
+        function_body = _find_first(function_definition, of_type=LangSexpVector)
+        rpy_fun_node = function_body[1]
+
+        assert str(rpy_fun_node[2]) == 'RPY2_FUN_PLACEHOLDER'
+        rpy_fun_node[2] = rpy_fun
+
     # TODO: use lower-level eval ?
     res = baseenv['eval'](template)
     # TODO: hack to prevent the nested function from having its
