@@ -1,28 +1,35 @@
 # TODO: make it cffi-buildable with conditional function definition
 # (Python if ABI, C if API)
+import abc
 import enum
+import inspect
 import logging
+from typing import Tuple
+import typing
 import warnings
-from . import ffi_proxy
-from . import openrlib
-from . import conversion
-from . import embedded
-from . import memorymanagement
+from rpy2.rinterface_lib import ffi_proxy
+from rpy2.rinterface_lib import openrlib
+from rpy2.rinterface_lib import conversion
+from rpy2.rinterface_lib import embedded
+from rpy2.rinterface_lib import memorymanagement
+
+from _cffi_backend import FFI  # type: ignore
+
 
 logger = logging.getLogger(__name__)
 
 ffi = openrlib.ffi
 
 # TODO: How can I reliably get MAX_INT from limits.h ?
-_MAX_INT = 2**32-1
+_MAX_INT: int = 2**32-1
 
-_R_PRESERVED = dict()
+_R_PRESERVED = dict()  # type: typing.Dict[int, int]
 _PY_PASSENGER = dict()
 
 FFI_MODE = ffi_proxy.get_ffi_mode(openrlib._rinterface_cffi)
 
 
-def get_rid(cdata) -> int:
+def get_rid(cdata: FFI.CData) -> int:
     """Get the identifier for the R object.
 
     This is intended to be like Python's `id()`, but
@@ -30,7 +37,7 @@ def get_rid(cdata) -> int:
     return int(ffi.cast('uintptr_t', cdata))
 
 
-def protected_rids():
+def protected_rids() -> Tuple[Tuple[int, int], ...]:
     """Sequence of R IDs protected from collection by rpy2."""
     keys = tuple(_R_PRESERVED.keys())
     res = []
@@ -41,16 +48,16 @@ def protected_rids():
     return tuple(res)
 
 
-def is_cdata_sexp(obj) -> bool:
+def is_cdata_sexp(obj: typing.Any) -> bool:
     """Is the object a cffi `CData` object pointing to an R object ?"""
-    if (isinstance(obj, ffi.CData) and
+    if (isinstance(obj, FFI.CData) and
             ffi.typeof(obj).cname == 'struct SEXPREC *'):
         return True
     else:
         return False
 
 
-def _preserve(cdata):
+def _preserve(cdata: FFI.CData) -> int:
     addr = int(ffi.cast('uintptr_t', cdata))
     count = _R_PRESERVED.get(addr, 0)
     if count == 0:
@@ -59,11 +66,11 @@ def _preserve(cdata):
     return addr
 
 
-def _release(cdata):
+def _release(cdata: FFI.CData) -> None:
     addr = int(ffi.cast('uintptr_t', cdata))
     count = _R_PRESERVED[addr] - 1
     if count == 0:
-        del(_R_PRESERVED[addr])
+        del _R_PRESERVED[addr]
         openrlib.rlib.R_ReleaseObject(cdata)
     else:
         _R_PRESERVED[addr] = count
@@ -71,14 +78,14 @@ def _release(cdata):
 
 @ffi_proxy.callback(ffi_proxy._capsule_finalizer_def,
                     openrlib._rinterface_cffi)
-def _capsule_finalizer(cdata):
+def _capsule_finalizer(cdata: FFI.CData) -> None:
     try:
         openrlib.rlib.R_ClearExternalPtr(cdata)
     except Exception as e:
         warnings.warn('Exception downgraded to warning: %s' % str(e))
 
 
-# ABI and API modes differs in the what is the exact callback object to be
+# ABI and API modes differs in what is the exact callback object to be
 # passed to C code.
 if hasattr(openrlib._rinterface_cffi, 'lib'):
     _capsule_finalizer_c = openrlib._rinterface_cffi.lib._capsule_finalizer
@@ -86,14 +93,12 @@ else:
     _capsule_finalizer_c = None
 
 
-class UnmanagedSexpCapsule(object):
+class CapsuleBase:
 
-    def __init__(self, cdata):
-        assert is_cdata_sexp(cdata)
-        self._cdata = cdata
+    _cdata: FFI.CData
 
     @property
-    def typeof(self):
+    def typeof(self) -> int:
         return _TYPEOF(self._cdata)
 
     @property
@@ -101,11 +106,34 @@ class UnmanagedSexpCapsule(object):
         return get_rid(self._cdata)
 
 
-class SexpCapsule(UnmanagedSexpCapsule):
+class UninitializedRCapsule(CapsuleBase):
+
+    @property
+    def _cdata(self):
+        raise RuntimeError('The embedded R is not initialized.')
+
+    @property
+    def typeof(self) -> int:
+        return self._typeof
+
+    def __init__(self, typeof):
+        self._typeof = typeof
+
+
+class UnmanagedSexpCapsule(CapsuleBase):
 
     __slots__ = ('_cdata', )
 
-    def __init__(self, cdata):
+    def __init__(self, cdata: FFI.CData):
+        assert is_cdata_sexp(cdata)
+        self._cdata = cdata
+
+
+class SexpCapsule(CapsuleBase):
+
+    __slots__ = ('_cdata', )
+
+    def __init__(self, cdata: FFI.CData):
         assert is_cdata_sexp(cdata)
         _preserve(cdata)
         self._cdata = cdata
@@ -126,7 +154,7 @@ class SexpCapsule(UnmanagedSexpCapsule):
 class SexpCapsuleWithPassenger(SexpCapsule):
     __slots__ = ('_cdata', '_passenger')
 
-    def __init__(self, cdata, passenger, ptr):
+    def __init__(self, cdata: FFI.CData, passenger, ptr):
         assert is_cdata_sexp(cdata)
         addr = _preserve(cdata)
         _PY_PASSENGER[addr] = passenger
@@ -137,7 +165,14 @@ class SexpCapsuleWithPassenger(SexpCapsule):
         addr = get_rid(self._cdata)
         _release(self._cdata)
         if addr not in _PY_PASSENGER:
-            del(_PY_PASSENGER[addr])
+            del _PY_PASSENGER[addr]
+
+
+class SupportsSEXP(object, metaclass=abc.ABCMeta):
+
+    @abc.abstractproperty
+    def __sexp__(self):
+        pass
 
 
 def _findvar(symbol, r_environment):
@@ -152,8 +187,58 @@ def _findfun(symbol, r_environment):
                            r_environment)
 
 
-def _findVarInFrame(symbol, r_environment):
-    return openrlib.rlib.Rf_findVarInFrame(r_environment, symbol)
+@ffi_proxy.callback(ffi_proxy._exec_findvar_in_frame_def,
+                    openrlib._rinterface_cffi)
+def _exec_findvar_in_frame(cdata):
+    cdata_struct = openrlib.ffi.cast('struct RPY2_sym_env_data *', cdata)
+    res = openrlib.rlib.Rf_findVarInFrame(
+        cdata_struct.environment,
+        cdata_struct.symbol
+    )
+    cdata_struct.data = res
+    return
+
+
+def findvar_in_frame(rho, symbol):
+    """Safer wrapper around Rf_findVarInFrame()
+
+    Run the function Rf_findVarInFrame() in R's C-API through
+    R_ToplevelExec().
+
+    Note: All arguments, and the object returned, are C-level
+    R objects.
+
+    Args:
+    - rho: An R environment.
+    - symbol: An R symbol (as returned by Rf_install())
+    Returns:
+    The object found.
+    """
+    # One would expect this to be like
+    #   res = _rinterface._findfun(symbol, self.__sexp__._cdata)
+    # but R's findfun will segfault if an error occurs while
+    # accessing the matching object in the environment.
+    exec_data = ffi.new(
+        'struct RPY2_sym_env_data *',
+        [symbol, rho, openrlib.rlib.R_NilValue]
+    )
+    _ = openrlib.rlib.R_ToplevelExec(
+        openrlib.rlib._exec_findvar_in_frame,
+        exec_data
+    )
+    if _ != openrlib.rlib.TRUE:
+        raise embedded.RRuntimeError(
+            'R C-API Rf_findVarInFrame()'
+        )
+    return exec_data.data
+
+
+if FFI_MODE is ffi_proxy.InterfaceType.ABI:
+    findvar_in_frame_wrap = openrlib.rlib.Rf_findVarInFrame
+elif FFI_MODE is ffi_proxy.InterfaceType.API:
+    findvar_in_frame_wrap = findvar_in_frame
+else:
+    raise ImportError('cffi mode unknown: %s' % FFI_MODE)
 
 
 def _GET_DIM(robj):
@@ -162,52 +247,57 @@ def _GET_DIM(robj):
     return res
 
 
-def _GET_DIMNAMES(robj):
+def _GET_DIMNAMES(robj) -> FFI.CData:
     res = openrlib.rlib.Rf_getAttrib(robj,
                                      openrlib.rlib.R_DimNames)
     return res
 
 
-def _GET_LEVELS(robj):
+def _GET_LEVELS(robj: FFI.CData) -> FFI.CData:
     res = openrlib.rlib.Rf_getAttrib(robj,
                                      openrlib.rlib.R_LevelsSymbol)
     return res
 
 
-def _GET_NAMES(robj):
+def _GET_NAMES(robj: FFI.CData) -> FFI.CData:
     res = openrlib.rlib.Rf_getAttrib(robj,
                                      openrlib.rlib.R_NamesSymbol)
     return res
 
 
-def _TYPEOF(robj):
+def _TYPEOF(robj: FFI.CData) -> int:
     return robj.sxpinfo.type
 
 
-def _SET_TYPEOF(robj, v):
+def _SET_TYPEOF(robj: FFI.CData, v: int):
     robj.sxpinfo.type = v
 
 
-def _NAMED(robj):
+def _NAMED(robj: FFI.CData) -> int:
     return robj.sxpinfo.named
 
 
-def _string_getitem(cdata, i):
-    rlib = openrlib.rlib
-    return conversion._cchar_to_str(
-        rlib.R_CHAR(rlib.STRING_ELT(cdata, i))
-    )
+def _string_getitem(cdata: FFI.CData, i: int) -> typing.Optional[str]:
+    elt = openrlib.rlib.STRING_ELT(cdata, i)
+    if elt == openrlib.rlib.R_NaString:
+        res = None
+    else:
+        res = conversion._cchar_to_str(
+            openrlib.rlib.R_CHAR(elt),
+            conversion._R_ENC_PY[openrlib.rlib.Rf_getCharCE(elt)]
+        )
+    return res
 
 
 # TODO: still used ?
-def _string_setitem(cdata, i, value_b):
+def _string_setitem(cdata: FFI.CData, i: int, CE: int, value_b) -> None:
     rlib = openrlib.rlib
     rlib.SET_STRING_ELT(
-        cdata, i, rlib.Rf_mkCharCE(value_b, conversion._CE_DEFAULT_VALUE)
+        cdata, i, rlib.Rf_mkCharCE(value_b, CE)
     )
 
 
-def _has_slot(cdata, name_b):
+def _has_slot(cdata: FFI.CData, name_b) -> bool:
     res = openrlib.rlib.R_has_slot(cdata, name_b)
     return bool(res)
 
@@ -250,7 +340,7 @@ def _evaluated_promise(function):
     return _
 
 
-def _python_index_to_c(cdata, i: int) -> int:
+def _python_index_to_c(cdata: FFI.CData, i: int) -> int:
     """Compute the C value for a Python-style index.
 
     The function will translate a Python-style index that
@@ -273,14 +363,14 @@ def _python_index_to_c(cdata, i: int) -> int:
 
 
 # TODO: make it a general check for names or symbols ?
-def _assert_valid_slotname(name: str):
+def _assert_valid_slotname(name: str) -> None:
     if not isinstance(name, str):
         raise ValueError('Invalid name %s' % repr(name))
     elif len(name) == 0:
         raise ValueError('The name cannot be an empty string')
 
 
-def _list_attrs(cdata):
+def _list_attrs(cdata: FFI.CData) -> FFI.CData:
     rlib = openrlib.rlib
     attrs = rlib.ATTRIB(cdata)
     nvalues = rlib.Rf_length(attrs)
@@ -316,7 +406,7 @@ def _list_attrs(cdata):
     return names
 
 
-def _remove(name, env, inherits):
+def _remove(name: FFI.CData, env: FFI.CData, inherits) -> FFI.CData:
     rlib = openrlib.rlib
     with memorymanagement.rmemory() as rmemory:
         internal = rmemory.protect(
@@ -331,7 +421,7 @@ def _remove(name, env, inherits):
     return res
 
 
-def _geterrmessage():
+def _geterrmessage() -> str:
     rlib = openrlib.rlib
     # TODO: use isString and installTrChar
     with memorymanagement.rmemory() as rmemory:
@@ -350,7 +440,7 @@ def _geterrmessage():
     return res
 
 
-def serialize(cdata, cdata_env):
+def serialize(cdata: FFI.CData, cdata_env: FFI.CData) -> FFI.CData:
     """Serialize an R object to an R string.
 
     Note that the R string returned is *not* protected from
@@ -376,7 +466,7 @@ def serialize(cdata, cdata_env):
         return res
 
 
-def unserialize(cdata, cdata_env):
+def unserialize(cdata: FFI.CData, cdata_env: FFI.CData) -> FFI.CData:
     """Unserialize an R string to an R object.
 
     Note that the R object returned is *not* protected from
@@ -406,13 +496,12 @@ def unserialize(cdata, cdata_env):
 
 @ffi_proxy.callback(ffi_proxy._evaluate_in_r_def,
                     openrlib._rinterface_cffi)
-def _evaluate_in_r(rargs):
+def _evaluate_in_r(rargs: FFI.CData) -> FFI.CData:
     # An uncaught exception in the boby of this function would
     # result in a segfault. we wrap it in a try-except an report
     # exceptions as logs.
 
     rlib = openrlib.rlib
-
     try:
         rargs = rlib.CDR(rargs)
         cdata = rlib.CAR(rargs)
@@ -423,21 +512,90 @@ def _evaluate_in_r(rargs):
             return rlib.R_NilValue
         handle = rlib.R_ExternalPtrAddr(cdata)
         func = ffi.from_handle(handle)
-
         pyargs = []
         pykwargs = {}
+        # R and Python function definitions differ. In R all arguments
+        # are optionally named, that is like "positional or keyword" arguments
+        # in Python. "positional or keyword" happens to be default for
+        # arguments without a default *unless* the ellipsis `*args` is used,
+        # in which case all arguments prior to it become positional-only.
+        # We will want to skip naming positional-only arguments although R
+        # might.
+        # TODO: Extraction information by inspecting the signature of the
+        # Python function will be done for each call. This is a performance burden.
+        # Is there a way to do it at "rternalization" time and access the result at
+        # call time?
+        # TODO: An other optimization would be to have a streamlined path for
+        # function that only have POSITIONAL_ONLY parameters (or may be
+        # POSITIONAL_ONLY and/or KEYWORD_ONLY).
+        pyfunc_params_iter = inspect.signature(func).parameters.items()
+        py_posonly = []
+        py_has_ellipsis = False
+        py_positionalorkw = []
+        for paramname, paramval in pyfunc_params_iter:
+            if paramval.kind is inspect.Parameter.POSITIONAL_ONLY:
+                py_posonly.append(paramname)
+            elif (paramval.kind is inspect.Parameter.VAR_POSITIONAL
+                  or
+                  paramval.kind is inspect.Parameter.VAR_KEYWORD):
+                py_has_ellipsis = True
+            elif paramval.kind is inspect.Parameter.POSITIONAL_OR_KEYWORD:
+                py_positionalorkw.append(paramname)
+            else:
+                # KEYWORD_ONLY from now on.
+                break
+        rarg_i = -1
         rargs = rlib.CDR(rargs)
         while rargs != rlib.R_NilValue:
+            rarg_i += 1
             cdata = rlib.CAR(rargs)
             if rlib.Rf_isNull(rlib.TAG(rargs)):
                 # Unnamed argument
                 pyargs.append(conversion._cdata_to_rinterface(cdata))
             else:
                 # Named arguments
+                rname = rlib.PRINTNAME(rlib.TAG(rargs))
                 name = conversion._cchar_to_str(
-                    rlib.R_CHAR(rlib.PRINTNAME(rlib.TAG(rargs)))
+                    rlib.R_CHAR(rname),
+                    conversion._R_ENC_PY[openrlib.rlib.Rf_getCharCE(rname)]
                 )
-                pykwargs[name] = conversion._cdata_to_rinterface(cdata)
+                if rarg_i < len(py_posonly):
+                    if py_posonly[rarg_i] == name:
+                        # This is an unnamed argument and names are matching.
+                        pyargs.append(conversion._cdata_to_rinterface(cdata))
+                    else:
+                        # The R call is considering the parameter as a named one
+                        # and the position is not conserved. This is can lead
+                        # to unnoticed issues, and difficult to debug ones when
+                        # they are. It is better to report the issue here.
+                        raise RuntimeError(
+                            'Parameter name mismatch. R call considering the argument '
+                            f'"{py_posonly[rarg_i]}" as a position-independent '
+                            'keyword argument while it is positional-only '
+                            'in the rternalized Python function.'
+                        )
+                elif (
+                        py_has_ellipsis
+                        and
+                        (rarg_i < (len(py_posonly) + len(py_positionalorkw)))
+                ):
+                    if py_positionalorkw[rarg_i - len(py_posonly)] == name:
+                        # This is considered an unnamed argument and names are matching.
+                        pyargs.append(conversion._cdata_to_rinterface(cdata))
+                    else:
+                        # The R call is considering the parameter as a named one
+                        # and the position is not conserved. This is can lead
+                        # to unnoticed issues, and difficult to debug ones when
+                        # they are. It is better to report the issue here.
+                        raise RuntimeError(
+                            'Parameter name mismatch. R call considering the argument '
+                            f'"{py_posonly[rarg_i]}" as a position-independent '
+                            'keyword argument while it is positional-or-keyword '
+                            'followed by an positional ellipsis `*args` in the '
+                            'rternalized Python function.'
+                        )
+                else:
+                    pykwargs[name] = conversion._cdata_to_rinterface(cdata)
             rargs = rlib.CDR(rargs)
 
         res = func(*pyargs, **pykwargs)
@@ -459,7 +617,7 @@ def _evaluate_in_r(rargs):
         else:
             return conversion._python_to_cdata(res)
     except Exception as e:
-        logger.error('%s: %s' % (type(e), e))
+        logger.error('%s: rternalized %s' % (type(e).__name__, e))
         return rlib.R_NilValue
 
 
@@ -499,14 +657,19 @@ class PARSING_STATUS(enum.Enum):
 
 class RParsingError(Exception):
 
-    def __init__(self, msg: str, status: int):
-        super().__init__(msg)
+    def __init__(self, msg: str,
+                 status: typing.Optional[PARSING_STATUS] = None):
+        full_msg = (
+            '{msg} - {status}'
+            .format(msg=msg, status=status)
+        )
+        super().__init__(full_msg)
         self.status = status
 
 
 @ffi_proxy.callback(ffi_proxy._parsevector_wrap_def,
                     openrlib._rinterface_cffi)
-def _parsevector_wrap(data):
+def _parsevector_wrap(data: FFI.CData):
     try:
         cdata, num, status = ffi.from_handle(data)
         res = openrlib.rlib.R_ParseVector(
@@ -520,6 +683,7 @@ def _parsevector_wrap(data):
     return res
 
 
+# TODO: is this complete ?
 @ffi_proxy.callback(ffi_proxy._handler_def,
                     openrlib._rinterface_cffi)
 def _handler_wrap(cond, hdata):
@@ -536,24 +700,23 @@ else:
     raise ImportError('cffi mode unknown: %s' % FFI_MODE)
 
 
-def _parse(cdata, num):
+def _parse(cdata: FFI.CData, num, rmemory) -> FFI.CData:
     status = ffi.new('ParseStatus[1]', None)
     data = ffi.new_handle((cdata, num, status))
     hdata = ffi.NULL
-    with memorymanagement.rmemory() as rmemory:
-        res = rmemory.protect(
-            openrlib.rlib.R_tryCatchError(
-                _parsevector_wrap, data,
-                _handler_wrap, hdata
-            )
+    res = rmemory.protect(
+        openrlib.rlib.R_tryCatchError(
+            _parsevector_wrap, data,
+            _handler_wrap, hdata
         )
-        # TODO: design better handling of possible status:
-        # PARSE_NULL,
-        # PARSE_OK,
-        # PARSE_INCOMPLETE,
-        # PARSE_ERROR,
-        # PARSE_EOF
-        if status[0] != openrlib.rlib.PARSE_OK:
-            raise RParsingError('Parsing status not OK',
-                                PARSING_STATUS(status[0]))
+    )
+    # TODO: design better handling of possible status:
+    # PARSE_NULL,
+    # PARSE_OK,
+    # PARSE_INCOMPLETE,
+    # PARSE_ERROR,
+    # PARSE_EOF
+    if status[0] != openrlib.rlib.PARSE_OK:
+        raise RParsingError('Parsing status not OK',
+                            status=PARSING_STATUS(status[0]))
     return res
