@@ -8,9 +8,10 @@ import warnings
 from collections import OrderedDict
 from rpy2.robjects.robject import RObjectMixin
 import rpy2.rinterface as rinterface
-from rpy2.rinterface_lib import na_values
+import rpy2.rinterface_lib.sexp
 from rpy2.robjects import help
 from rpy2.robjects import conversion
+from rpy2.robjects.vectors import Vector
 
 from rpy2.robjects.packages_utils import (default_symbol_r2python,
                                           default_symbol_resolve,
@@ -27,11 +28,13 @@ __is_null = baseenv_ri.find('is.null')
 
 
 def _formals_fixed(func):
-    tmp = __args(func)
-    if __is_null(tmp)[0]:
-        return rinterface.NULL
+    if func.typeof is rpy2.rinterface_lib.sexp.RTYPES.SPECIALSXP:
+        res = rpy2.rinterface_lib.sexp.NULL
     else:
-        return __formals(tmp)
+        res = __formals(func)
+        if res is rpy2.rinterface_lib.sexp.NULL:
+            res = __formals(__args(func))
+    return res
 
 
 # docstring_property and DocstringProperty
@@ -107,23 +110,25 @@ class Function(RObjectMixin, rinterface.SexpClosure):
                     'R arguments:', ''])
         if fm is rinterface.NULL:
             doc.append('<No information available>')
-        for key, val in zip(fm.do_slot('names'), fm):
-            if key == '...':
-                val = 'R ellipsis (any number of parameters)'
-            doc.append('%s: %s' % (key, _repr_argval(val)))
+        else:
+            for key, val in zip(fm.do_slot('names'), fm):
+                if key == '...':
+                    val = 'R ellipsis (any number of parameters)'
+                doc.append('%s: %s' % (key, _repr_argval(val)))
         return os.linesep.join(doc)
 
     def __call__(self, *args, **kwargs):
-        new_args = [conversion.py2rpy(a) for a in args]
+        cv = conversion.get_conversion()
+        new_args = [cv.py2rpy(a) for a in args]
         new_kwargs = {}
         for k, v in kwargs.items():
             # TODO: shouldn't this be handled by the conversion itself ?
             if isinstance(v, rinterface.Sexp):
                 new_kwargs[k] = v
             else:
-                new_kwargs[k] = conversion.py2rpy(v)
+                new_kwargs[k] = cv.py2rpy(v)
         res = super(Function, self).__call__(*new_args, **new_kwargs)
-        res = conversion.rpy2py(res)
+        res = cv.rpy2py(res)
         return res
 
     def formals(self):
@@ -131,12 +136,13 @@ class Function(RObjectMixin, rinterface.SexpClosure):
         (as the R function 'formals()' would).
         """
         res = _formals_fixed(self)
-        res = conversion.rpy2py(res)
+        res = conversion.get_conversion().rpy2py(res)
         return res
 
-    def rcall(self,
-              keyvals,
-              environment: typing.Optional[rinterface.SexpEnvironment] = None
+    def rcall(
+            self,
+            keyvals,
+            environment: typing.Optional[rinterface.SexpEnvironment] = None
     ) -> rinterface.sexp.Sexp:
         """ Wrapper around the parent method
         rpy2.rinterface.SexpClosure.rcall(). """
@@ -181,12 +187,15 @@ class SignatureTranslatedFunction(Function):
                              msg_prefix,
                              exception)
             symbol_mapping.update(resolutions)
-            reserved_pynames = set(dir(self))
+            # TODO: Why was this done?
+            # reserved_pynames = set(dir(self))
 
             self._prm_translate.update((k, v[0])
                                        for k, v in symbol_mapping.items())
         if hasattr(sexp, '__rname__'):
-            self.__rname__ = sexp.__rname__
+            # TODO: mypy does not use the line above and trips on
+            # __rname__ being not always present.
+            self.__rname__ = sexp.__rname__  # type: ignore
 
     def __call__(self, *args, **kwargs):
         prm_translate = self._prm_translate
@@ -216,18 +225,27 @@ class DocumentedSTFunction(SignatureTranslatedFunction):
 
     @docstring_property(__doc__)
     def __doc__(self):
+        package = help.Package(self.__rpackagename__)
+        page = package.fetch(self.__rname__)
+
         doc = ['Wrapper around an R function.',
                '',
                'The docstring below is built from the R documentation.',
                '']
 
-        description = help.docstring(self.__rpackagename__,
-                                     self.__rname__,
-                                     sections=['\\description'])
-        doc.append(description)
+        if r'\description' in page.sections:
+            doc.append(
+                page.to_docstring(
+                    section_names=[r'\description']
+                )
+            )
 
         fm = _formals_fixed(self)
-        names = fm.do_slot('names')
+        if fm is rpy2.rinterface_lib.sexp.NULL:
+            # If still NULL there is no argument.
+            names = tuple()
+        else:
+            names = fm.do_slot('names')
         doc.append(self.__rname__+'(')
         for key, val in self._prm_translate.items():
             if key == '___':
@@ -240,18 +258,19 @@ class DocumentedSTFunction(SignatureTranslatedFunction):
             else:
                 doc.append('    %s = %s,' % (key, description))
         doc.extend((')', ''))
-        package = help.Package(self.__rpackagename__)
-        page = package.fetch(self.__rname__)
+        ####
         doc.append('Args:')
         for item in page.arguments():
             description = ('%s  ' % os.linesep).join(item.value)
             doc.append(' '.join(('  ', item.name, ': ', description)))
             doc.append('')
 
-        doc.append(help.docstring(self.__rpackagename__,
-                                  self.__rname__,
-                                  sections=['\\details'])
-        )
+        if r'\details' in page.sections:
+            doc.append(
+                page.to_docstring(
+                    section_names=[r'\details']
+                )
+            )
 
         return os.linesep.join(doc)
 
@@ -267,14 +286,19 @@ def _map_default_value(value: rinterface.Sexp):
     """
     Map default in the R signature.
 
-    Because of R's lazy evaluation some default might be unevaluated expressions.
+    Because of R's lazy evaluation some default might be unevaluated
+    expressions.
 
     Args:
       value:
     """
     if value.__sexp__.typeof in _SCALAR_COMPAT_RTYPES:
-        if len(value) == 1:
-            res = value[0]
+        # TODO: The dynamic check of typeof (to ensure that that
+        # the underlying R object is of a compatible type makes
+        # mypy trip. There is no way to check type outside of runtime.
+        # Code refactoring would be needed.
+        if len(value) == 1:  # type: ignore
+            res = value[0]  # type: ignore
         else:
             res = value
     else:
@@ -312,16 +336,21 @@ def map_signature(
     r_params = r_func.formals()
     rev_prm_transl = {v: k for k, v in r_func._prm_translate.items()}
     if r_params.names is not rinterface.NULL:
-        for i, (name, default_orig) in enumerate(zip(r_params.names, r_params)):
+        for i, (name, default_orig) in enumerate(
+                zip(r_params.names, r_params)
+        ):
             if default_orig == '...':
                 r_ellipsis = i
                 warnings.warn('The R ellispsis is not yet well supported.')
             transl_name = rev_prm_transl.get(name)
-            default_orig = default_orig[0]
-            if map_default and not rinterface.MissingArg.rsame(default_orig):
-                default_mapped = map_default(default_orig)
+            if isinstance(default_orig, Vector):
+                default_orig = default_orig[0]
+                if map_default and not rinterface.MissingArg.rsame(default_orig):
+                    default_mapped = map_default(default_orig)
+                else:
+                    default_mapped = inspect.Parameter.empty
             else:
-                default_mapped = inspect.Parameter.empty
+                default_mapped = default_orig
             prm = inspect.Parameter(
                 transl_name if transl_name else name,
                 inspect.Parameter.POSITIONAL_OR_KEYWORD,
@@ -346,8 +375,8 @@ def wrap_docstring_default(
         is_method (bool): Whether the function should be treated as a method
             (a `self` parameter is added to the signature if so).
         signature (inspect.Signature): A mapped signature for `r_func`
-        r_ellipsis (bool): Index of the parameter containing the R ellipsis (`...`).
-            None if the R ellipsis is not in the function signature.
+        r_ellipsis (bool): Index of the parameter containing the R ellipsis
+            (`...`). None if the R ellipsis is not in the function signature.
         full_repr (bool): Whether to have the full body of the R function in
             the docstring dynamically generated.
     Returns:
@@ -364,9 +393,9 @@ def wrap_docstring_default(
              textwrap.dedent(
                  """The R ellipsis "..." present in the function's parameters
                  is mapped to a python iterable of (name, value) pairs (such as
-                 it is returned by the `dict` method `items()` for example."""),
-             ''
-            )
+                 it is returned by the `dict` method `items()` for example."""
+             ),
+             '')
         )
     if full_repr:
         docstring.append('\n{}'.format(r_func.r_repr()))
@@ -377,7 +406,7 @@ def wrap_docstring_default(
             docstring.append('\n{}'.format(r_func.r_repr()))
         else:
             docstring.append('\n{}\n{{\n  ...\n}}'.format(r_repr[:i]))
-    
+
     return '\n'.join(docstring)
 
 
@@ -397,7 +426,7 @@ def wrap_r_function(
             ] = wrap_docstring_default
 ) -> typing.Callable:
     """
-    Wrap an rpy2 function handle with a Python function with a matching signature.
+    Wrap an rpy2 function handle with a Python function of matching signature.
 
     Args:
         r_func (rpy2.robjects.functions.SignatureTranslatedFunction): The
@@ -417,10 +446,12 @@ def wrap_r_function(
 
     if r_ellipsis:
         def wrapped_func(*args, **kwargs):
-            new_args = (list((None, x) for x in rinterface.args[:r_ellipsis]) +
-                        list(args[r_ellipsis]) +
-                        list((None, x) for x in args[min(r_ellipsis+1, len(args)-1):]) +
-                        list(kwargs.items()))
+            new_args = (
+                list((None, x) for x in rinterface.args[:r_ellipsis]) +
+                list(args[r_ellipsis]) +
+                list((None, x)
+                     for x in args[min(r_ellipsis+1, len(args)-1):]) +
+                list(kwargs.items()))
             value = r_func.rcall(new_args)
             return value
     else:
@@ -435,7 +466,10 @@ def wrap_r_function(
 
     wrapped_func.__name__ = name
     wrapped_func.__qualname__ = name
-    wrapped_func.__signature__ = signature
+    # TODO: Open issue in mypy about __signature.
+    # https://github.com/python/mypy/issues/5958
+    # Ignore the type check for now.
+    wrapped_func.__signature__ = signature  # type: ignore
     wrapped_func.__doc__ = docstring
 
     return wrapped_func
