@@ -6,11 +6,12 @@ with warnings.catch_warnings():
     warnings.simplefilter("ignore")
     tidyr = importr('tidyr', on_conflict="warn")
 
-TARGET_VERSION = '0.8.3'
+TARGET_VERSION = '1.1.'
 
-if tidyr.__version__ != TARGET_VERSION:
-    warnings.warn('This was designed againt tidyr version %s '
-                  'but you have %s' % (TARGET_VERSION, tidyr.__version__))
+if not tidyr.__version__.startswith(TARGET_VERSION):
+    warnings.warn(
+        'This was designed againt tidyr versions starting with %s '
+        'but you have %s' % (TARGET_VERSION, tidyr.__version__))
 
 tidyr = WeakPackage(tidyr._env,
                     tidyr.__rname__,
@@ -33,7 +34,7 @@ def _wrap(rfunc):
 class DataFrame(dplyr.DataFrame):
     gather = _wrap(tidyr.gather)
     spread = _wrap(tidyr.spread)
+    summarize = dplyr._wrap(dplyr.summarize, None)
 
 
-DataFrame.summarize = dplyr._wrap(dplyr.summarize, None)
 DataFrame.summarise = DataFrame.summarize
